## Copyright (C) 2008 Bill Denney <bill@denney.ws>
##
## This program is free software; you can redistribute it and/or modify it under
## the terms of the GNU General Public License as published by the Free Software
## Foundation; either version 3 of the License, or (at your option) any later
## version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, see <http://www.gnu.org/licenses/>.

## -*- texinfo -*-
## @deftypefn {Function File} {datenum =} today ()
## Returns the current local date as the number of days since Jan 1, 0000.
## By this reckoning, Jan 1, 1970 is day number 719529.
##
## The returned number corresponds to 00:00:00 today.
##
## The returned value is also called a "serial date number"
## (see @code{datenum}).
## @seealso{clock, date, datenum, now}
## @end deftypefn

function t = today ()

  t = floor (now ());

endfunction
